using System;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Collections.Generic;
using Sample;

namespace Sample {
	public class Window : Form {
		private const int width = 640;
		private const int height = 480;
		private Device device;
		private Sprite sprite;
		
		private TextureImageSprite image1, image2;
		private int a = 0;
		private int am = 8;
		private int x = 0;
		private int xm = 8;
		private int y = 0;
		private int ym = 4;
		
		public Window() {
			Text = "Direct3D with Managed DirectX";
			FormBorderStyle = FormBorderStyle.Fixed3D;
			MaximizeBox = false;
			StartPosition = FormStartPosition.CenterScreen;
			ClientSize = new Size(width, height);
			BackColor = Color.White;
			SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque, true);
			
			DisplayMode mode = Manager.Adapters[0].CurrentDisplayMode;
			Format format = mode.Format;
			
			PresentParameters pp = new PresentParameters();
			pp.Windowed = false;
			pp.BackBufferCount = 1;
			pp.BackBufferFormat = format;
			pp.BackBufferWidth = width;
			pp.BackBufferHeight = height;
			pp.SwapEffect = SwapEffect.Discard;
			
			try {
				device = new Device(0, DeviceType.Hardware, this, CreateFlags.HardwareVertexProcessing, pp);
			} catch (DirectXException) {
				try {
					device = new Device(0, DeviceType.Hardware, this, CreateFlags.SoftwareVertexProcessing, pp);
				} catch (DirectXException) {
					try {
						device = new Device(0, DeviceType.Reference, this, CreateFlags.SoftwareVertexProcessing, pp);
					} catch (DirectXException e) {
						MessageBox.Show(e.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
						Close();
					}
				}
			}
			
			System.Windows.Forms.Cursor.Hide();
			
			sprite = new Sprite(device);
			
			Bitmap bitmap = new Bitmap("ball.bmp");
			bitmap.MakeTransparent(Color.FromArgb(0x00, 0x00, 0xFF));
			image1 = new TextureImageSprite(device, sprite, bitmap);
			image2 = new TextureImageSprite(device, sprite, "back.bmp");
		}
		
		public void FrameMove() {
			x += xm;
			if (x > 460 || x < 0) {
				xm = xm * -1;
			}
			y += ym;
			if (y > 300 || y < 0) {
				ym = ym * -1;
			}
			a += am;
			if (a >= 255 || a <= 0) {
				if (am > 0) a = 255;
				else a = 0;
				am = am * -1;
			}
		}
		
		protected override void OnPaint(PaintEventArgs e) {
			base.OnPaint(e);
			
			device.Clear(ClearFlags.Target, Color.Black, 0, 0);
			device.BeginScene();
			sprite.Begin(SpriteFlags.AlphaBlend);
			image2.Draw2D(0, 0);
			image1.Draw2D(x, y, a);
			sprite.End();
			device.EndScene();
			device.Present();
		}
		
		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (device != null)  {
					device.Dispose();
				}
			}
			base.Dispose(disposing);
		}
		
		protected override void OnKeyDown(KeyEventArgs e) {
			base.OnKeyDown(e);
			if (e.KeyCode == Keys.Escape) {
				Close();
			}
		}
	}
}
